#ifndef __DSC_CONNECT_H
#define __DSC_CONNECT_H 1

#define SUPPORT_WINDOWS_98SE
#define SUPPORT_SERIAL_COM
#define DEBUG

#ifdef DEBUG

bool open_log(const char*);
void close_log();
void __cdecl dprintf(const char*,...);

#define DOUT(x) { dprintf x; }

#else

#define DOUT(x) {;}

#endif

typedef unsigned long dword;
typedef unsigned short word;
typedef unsigned char byte;

struct TDeviceInfo
{
    byte VendorId[10];
    byte ProductId[18];
    byte ProductRevisionLevel[6];
};

class TResponce
{
public:
   ~TResponce() { free(); }
   TResponce() { ptr=0; size=0; }

   bool alloc(unsigned sz) { free(); return 0 != (ptr = new char[size=sz]); }
   void free() { delete [] ptr; ptr=0; size=0; }
   bool set(const void* data, unsigned size);
   bool append(const void* data, unsigned size);

   char* ptr;
   int size;

   const char* PrepareStr();
   const char* GetStr() const { return (const char*)ptr; }
   int GetSize() const { return size; }
};

class DSC
{
public:

   DSC(const char* comment);
   virtual ~DSC();

   virtual bool SetReg(int reg, unsigned value) const = 0;
   virtual bool GetReg(int reg, unsigned& value) const = 0;

   virtual bool SetVar(int reg, const void* data, unsigned size) const = 0;
   virtual bool SetStr(int reg, const char*) const;
   virtual bool GetVar(int reg, TResponce&) const = 0;

   virtual bool SpecCmd(int reg, unsigned value, unsigned timeout=0) const = 0;

   virtual bool GetDeviceInfo(TDeviceInfo&) const = 0;

   enum { PACKET_SIZE = 2048 };
   char comment[8];
};

enum {
   PARAM_COM = 1 << 24,
   PARAM_USB_ASPI = 2 << 24,
   PARAM_USB_IOCTL = 3 << 24,
   PARAM_TYPE = 0x0F << 24,
};

DSC* OpenUSB();
DSC* OpenUSB(char camera_drive);
bool CheckUSB(char, char*, unsigned);

struct IOCTL_DriveName
{
   IOCTL_DriveName() : i(-1) {}
   void Rewind() { i = -1; }

   int i;
   unsigned drives;
   char drive[4];

   const char* GetNext(unsigned&);
};

#ifdef SUPPORT_WINDOWS_98SE

DSC* OpenUSB_ASPI();
DSC* OpenUSB_ASPI(int ha, int id, int lun);
bool CheckUSB_ASPI(int ha, int id, int lun, char*, unsigned);

struct ASPI_HostName
{
   ASPI_HostName() : HaId(-1) {}
   int HaId, SCSI_Id, Lun, nHosts;
   char host[20];

   const char* GetNext(unsigned&);
   void Rewind() { HaId = -1; }
};

#endif

#ifdef SUPPORT_SERIAL_COM

DSC* OpenCOM();
DSC* OpenCOM(char camera_drive);
bool CheckCOM(char, char*, unsigned);

struct COM_PortName
{
   COM_PortName() : i(-1) {}
   int i;
   char port[8];

   const char* GetNext(unsigned&);
   void Rewind() { i = -1; }
};

#endif


#endif//#ifndef __DSC_CONNECT_H

